//
//  TDLevelLayer.m
//  Viral Zombies TD
//
//  Created by Yo on 23/11/11.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import "TDLevelLayer.h"
#import "GameHUD.h"

#import "DataModel.h"
#import "GameManager.h"

#import "Constants.h"
#import "Goal.h"

@implementation TDLevelLayer

@synthesize tileMap = _tileMap;
@synthesize background = _background;

@synthesize difficulty = _difficulty;
@synthesize currentLevel = _currentLevel;
@synthesize currentWave = _currentWave;
@synthesize isPause = _isPause;
@synthesize maxScore = _maxScore;

- (void)displayScoreCoins {
    DataModel *m = [DataModel getModel];
    [labelScore setString:[NSString stringWithFormat:@"Score:%05d", m._currentScore]];  
    [labelCoins setString:[NSString stringWithFormat:@"%05d$", m._currentCoins]];    
    [labelHP setString:[NSString stringWithFormat:@"Live:%0d", m._currentHP]];   
}

// on "init" you need to initialize your instance
-(id) init {
    if((self = [super init])) {	
        self.isPause = false;
        DataModel *m = [DataModel getModel];
        CGSize winSize = [CCDirector sharedDirector].winSize;
        [m reset];
        
        NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
        self.maxScore = [defaults doubleForKey:@"HighScores"];
        switch ([[GameManager sharedGameManager] difficulty]) {
            case 0:
                self.difficulty = 0.5;
                break;
            case 1:
                self.difficulty = 0.75;
                break;
            case 2:
                self.difficulty = 1.0;
                break;
            case 3:
                self.difficulty = 1.5;
                break;
            default:
                break;
        }
        
        self.currentLevel = [[GameManager sharedGameManager] currentLevel];
        [defaults synchronize];
        Goal *levelGoal = nil;
        
        switch (self.currentLevel) {
            case 1:
                self.tileMap = [CCTMXTiledMap tiledMapWithTMXFile:@"Level1.tmx"];
                levelGoal = [LungGoal goal];
                levelGoal.position = ccp(winSize.width-20, winSize.height*0.5);
                m._currentCoins = 500;
                break;
            case 2:
                self.tileMap = [CCTMXTiledMap tiledMapWithTMXFile:@"Level2.tmx"];
                levelGoal = [HeartGoal goal];
                levelGoal.position = ccp(winSize.width-20, winSize.height*0.3);
                m._currentCoins = 500;
                break;
            case 3:
                self.tileMap = [CCTMXTiledMap tiledMapWithTMXFile:@"Level3.tmx"];
                levelGoal = [BrainGoal goal];
                levelGoal.position = ccp(winSize.width-20, winSize.height*0.2);
                m._currentCoins = 500;
                break;
            default:
                break;
        }
        
        m._gameGoal = levelGoal;
        [self addChild:levelGoal z:1];
        
		//self.tileMap = [CCTMXTiledMap tiledMapWithTMXFile:@"TileMap.tmx"];
        self.background = [_tileMap layerNamed:@"Background"];
		self.background.anchorPoint = ccp(0, 0);
		[self addChild:_tileMap z:0];
		
        gameHUD = [GameHUD sharedHUD];
        [self addChild:gameHUD z:2];
        
        m._gameLayer = self;
        m._gameHUDLayer = gameHUD;
        
		[self addWaypoint];
		[self addWaves];
		
		// Call game logic about every second
        [self schedule:@selector(update:)];
		[self schedule:@selector(gameLogic:) interval:1.0];		
        
		self.currentWave = 0;
		self.position = ccp(0, 0);
		
        
        float fontSize = 20.0;
        if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
            fontSize *= 2;
        }
        
        labelScore = [CCLabelTTF labelWithString:@"" fontName:@"AmericanTypewriter-Bold" fontSize:fontSize];
        labelScore.anchorPoint = ccp(1, 1);
        labelScore.position = ccp(winSize.width - 350, winSize.height);
        [self addChild:labelScore]; 
        
        labelCoins = [CCLabelTTF labelWithString:@"" fontName:@"AmericanTypewriter-Bold" fontSize:fontSize];
        labelCoins.anchorPoint = ccp(1, 1);
        labelCoins.position = ccp(winSize.width, winSize.height);
        [self addChild:labelCoins]; 
        
        labelHP = [CCLabelTTF labelWithString:@"" fontName:@"AmericanTypewriter-Bold" fontSize:fontSize];
        labelHP.anchorPoint = ccp(1, 1);
        labelHP.position = ccp(winSize.width-120, winSize.height);
        [self addChild:labelHP]; 
        
        CCLabelBMFont *pauseSceneLabel = [CCLabelTTF labelWithString:@"||" fontName:@"AmericanTypewriter-Bold" fontSize:fontSize];
        CCMenuItemLabel *gamePauseMenu = [CCMenuItemLabel itemWithLabel:pauseSceneLabel target:self selector:@selector(pauseGame)];
        [gamePauseMenu setTag:2];
        
        gameMenu = [CCMenu menuWithItems:gamePauseMenu, nil];
        [gameMenu setPosition:ccp(winSize.width*0.4, winSize.height-12)];

        [gameMenu alignItemsHorizontally];
        [self addChild:gameMenu];
        
        [self displayScoreCoins];
		
        [[GameManager sharedGameManager] playBackgroundTrack:BACKGROUND_TRACK_LEVEL];
    }
    return self;
}

-(void)addWaves {
	DataModel *m = [DataModel getModel];
    Wave *wave = nil;
    switch (self.currentLevel) {
        case 1:
            wave = [[Wave alloc] initWithCreep:kNormalCreep SpawnRate:1.0 TotalCreeps:5];
            [m._waves addObject:wave];
            wave = [[Wave alloc] initWithCreep:kNormalCreep SpawnRate:1.0 TotalCreeps:10];
            [m._waves addObject:wave];
            wave = [[Wave alloc] initWithCreep:kNormalCreep SpawnRate:1.0 TotalCreeps:15];
            [m._waves addObject:wave];
            wave = [[Wave alloc] initWithCreep:kNormalCreep SpawnRate:1.0 TotalCreeps:20];
            [m._waves addObject:wave];
            wave = [[Wave alloc] initWithCreep:kNormalCreep SpawnRate:1.0 TotalCreeps:25];
            [m._waves addObject:wave];
            break;
        case 2:
            wave = [[Wave alloc] initWithCreep:kNormalCreep SpawnRate:1.0 TotalCreeps:10];
            [m._waves addObject:wave];
            wave = [[Wave alloc] initWithCreep:kFastCreep SpawnRate:1.0 TotalCreeps:10];
            [m._waves addObject:wave];
            wave = [[Wave alloc] initWithCreep:kNormalCreep SpawnRate:1.0 TotalCreeps:15];
            [m._waves addObject:wave];
            wave = [[Wave alloc] initWithCreep:kNormalCreep SpawnRate:1.0 TotalCreeps:15];
            [m._waves addObject:wave];
            wave = [[Wave alloc] initWithCreep:kFastCreep SpawnRate:1.0 TotalCreeps:10];
            [m._waves addObject:wave];
            break;
        case 3:
            wave = [[Wave alloc] initWithCreep:kNormalCreep SpawnRate:1.0 TotalCreeps:20];
            [m._waves addObject:wave];
            wave = [[Wave alloc] initWithCreep:kStrongCreep SpawnRate:1.0 TotalCreeps:10];
            [m._waves addObject:wave];
            wave = [[Wave alloc] initWithCreep:kFastCreep SpawnRate:1.0 TotalCreeps:20];
            [m._waves addObject:wave];
            wave = [[Wave alloc] initWithCreep:kNormalCreep SpawnRate:1.0 TotalCreeps:20];
            [m._waves addObject:wave];
            wave = [[Wave alloc] initWithCreep:kStrongCreep SpawnRate:1.0 TotalCreeps:20];
            [m._waves addObject:wave];
            break;
        default:
            break;
    }
    wave = nil;	
}

- (Wave *)getCurrentWave{
	
	DataModel *m = [DataModel getModel];	
	Wave * wave = (Wave *) [m._waves objectAtIndex:self.currentWave];
	
	return wave;
}

- (Wave *)getNextWave{
	
	DataModel *m = [DataModel getModel];
	
	self.currentWave++;
	
	if (self.currentWave > m._waves.count - 1)
		self.currentWave = m._waves.count - 1;
	
    Wave * wave = (Wave *) [m._waves objectAtIndex:self.currentWave];
    
    return wave;
}

-(void)addWaypoint {
	DataModel *m = [DataModel getModel];
	
	CCTMXObjectGroup *objects = [self.tileMap objectGroupNamed:@"Objects"];
	WayPoint *wp = nil;
	
	int wayPointCounter = 0;
	NSMutableDictionary *wayPoint;
	while ((wayPoint = [objects objectNamed:[NSString stringWithFormat:@"Waypoint%d", wayPointCounter]])) {
		int x = [[wayPoint valueForKey:@"x"] intValue];
		int y = [[wayPoint valueForKey:@"y"] intValue];
		
		wp = [WayPoint node];
		wp.position = ccp(x, y);
		[m._waypoints addObject:wp];
		wayPointCounter++;
	}
	
	NSAssert([m._waypoints count] > 0, @"Waypoint objects missing");
	wp = nil;
}

- (CGPoint) tileCoordForPosition:(CGPoint) position 
{
	int x = position.x / self.tileMap.tileSize.width;
	int y = ((self.tileMap.mapSize.height * self.tileMap.tileSize.height) - position.y) / self.tileMap.tileSize.height;
	
	return ccp(x,y);
}

- (BOOL) canBuildOnTilePosition:(CGPoint) pos 
{
	CGPoint towerLoc = [self tileCoordForPosition: pos];
	
	int tileGid = [self.background tileGIDAt:towerLoc];
	NSDictionary *props = [self.tileMap propertiesForGID:tileGid];
	NSString *type = [props valueForKey:@"buildable"];
	
	if([type isEqualToString: @"1"]) {
		return YES;
	}
	
	return NO;
}

-(void)addTower: (CGPoint)pos, int tag {
	DataModel *m = [DataModel getModel];
	
	Tower *target = nil;
	
	CGPoint towerLoc = [self tileCoordForPosition: pos];
	
	int tileGid = [self.background tileGIDAt:towerLoc];
	NSDictionary *props = [self.tileMap propertiesForGID:tileGid];
	NSString *type = [props valueForKey:@"buildable"];
    
	NSLog(@"Buildable: %@", type);
    CGPoint posicion = ccp((towerLoc.x * 32) + 16, self.tileMap.contentSize.height - (towerLoc.y * 32) - 16);
    
    bool IsEmpty = true;
    
    for (Tower *tower in m._towers) {
        float distance = ccpDistance(tower.position,posicion);
		if(distance == 0)
            IsEmpty = false;
    }
    
	if([type isEqualToString: @"1"] && IsEmpty) {
        switch(tag)
        {
            case 1:
                target = [MachineGunTower tower];
                if(m._currentCoins >=[target cost])
                {
                    target.position = posicion;
                    [self addChild:target z:1];
                    m._currentCoins -= [target cost];
                    target.tag = 1;
                    [m._towers addObject:target];
                    
                }
                break;
            case 2:
                target = [SplashTower tower];
                if(m._currentCoins >=[target cost])
                {
                    target.position = posicion;
                    [self addChild:target z:1];
                    m._currentCoins -= [target cost];
                    target.tag = 2;
                    [m._towers addObject:target];
                }
                break;
            case 3:
                target = [FrostTower tower];
                if(m._currentCoins >=[target cost])
                {
                    target.position = posicion;
                    [self addChild:target z:1];
                    m._currentCoins -= [target cost];
                    target.tag = 3;
                    [m._towers addObject:target];
                }
                break;
            case 4:
                target = [CoinTower tower];
                if(m._currentCoins >=[target cost])
                {
                    target.position = ccp((towerLoc.x * 32) + 16, self.tileMap.contentSize.height - (towerLoc.y * 32) - 16);
                    [self addChild:target z:1];
                    m._currentCoins -= [target cost];
                    target.tag = 4;
                    [m._towers addObject:target];
                }
                break;
        }
	} else {
		NSLog(@"Tile Not Buildable");
	}
	[self displayScoreCoins];
}

-(void)addTarget {
    
	DataModel *m = [DataModel getModel];
	Wave * wave = [self getCurrentWave];
    
    
	if (wave.totalCreeps <= 0) {
        [self getNextWave];
		return;
	}
	
	wave.totalCreeps--;
	
    Creep *target = nil;
    
    switch(wave.creepType)
    {
        case kNormalCreep:
            target = [NormalCreep creep];
            break;
        case kFastCreep:
            target = [FastCreep creep];
            break;
        case kStrongCreep:
            target = [StrongCreep creep];
            break;
    }
    
    double factor2 = self.currentWave/5;
    double factor = self.difficulty+factor2;
    target.hp = target.hp * factor;
    
    
	WayPoint *waypoint = [target getCurrentWaypoint ];
	target.position = waypoint.position;	
	waypoint = [target getNextWaypoint ];
	
	[self addChild:target z:1];
	
    float distance = ccpDistance(target.position,waypoint.position);
    float moveDuration = (distance*target.moveDuration)/400.0;
    
	id actionMove = [CCMoveTo actionWithDuration:moveDuration position:waypoint.position];
	id actionMoveDone = [CCCallFuncN actionWithTarget:self selector:@selector(FollowPath:)];
	[target runAction:[CCSequence actions:actionMove, actionMoveDone, nil]];
	
	// Add to targets array
	target.tag = 1;
	[m._targets addObject:target];
	
}

-(void)FollowPath:(id)sender {
    
	Creep *creep = (Creep *)sender;
	
	WayPoint * waypointA = [creep getCurrentWaypoint];
	WayPoint * waypointN = [creep getNextWaypoint];
    
    if(waypointA != waypointN)
    {
        float distance = ccpDistance(creep.position,waypointN.position);
        float moveDuration = (distance*creep.moveDuration)/400.0;
        
        id actionMove = [CCMoveTo actionWithDuration:moveDuration position:waypointN.position];
        id actionMoveDone = [CCCallFuncN actionWithTarget:self selector:@selector(FollowPath:)];
        [creep stopAllActions];
        [creep runAction:[CCSequence actions:actionMove, actionMoveDone, nil]];
    }
    else
    {
        DataModel *m = [DataModel getModel];
        m._currentHP -= creep.damage;
        [m._targets removeObject:creep];
        [self removeChild:creep cleanup:YES];	
        [self displayScoreCoins];
        if(m._currentHP <= 0)
            [self pauseGame];
    }
}

-(void)gameLogic:(ccTime)dt {
	
	DataModel *m = [DataModel getModel];
	Wave * wave = [self getCurrentWave];
	static double lastTimeTargetAdded = 0;
    double now = [[NSDate date] timeIntervalSince1970];
    if(lastTimeTargetAdded == 0 || now - lastTimeTargetAdded >= wave.spawnRate) {
        [self addTarget];
        lastTimeTargetAdded = now;
    }
    
    [self displayScoreCoins];
}

- (void)update:(ccTime)dt {
    
	DataModel *m = [DataModel getModel];
	NSMutableArray *projectilesToDelete = [[NSMutableArray alloc] init];
    
	for (Projectile *projectile in m._projectiles) {
		
		CGRect projectileRect = CGRectMake(projectile.position.x - (projectile.contentSize.width/2), 
										   projectile.position.y - (projectile.contentSize.height/2), 
										   projectile.contentSize.width, 
										   projectile.contentSize.height);
        
		NSMutableArray *targetsToDelete = [[NSMutableArray alloc] init];
		
		for (CCSprite *target in m._targets) {
			CGRect targetRect = CGRectMake(target.position.x - (target.contentSize.width/2), 
										   target.position.y - (target.contentSize.height/2), 
										   target.contentSize.width, 
										   target.contentSize.height);
            
			if (CGRectIntersectsRect(projectileRect, targetRect)) {
                
				[projectilesToDelete addObject:projectile];
				
                Creep *creep = (Creep *)target;
                switch (creep.tag) {
                    case kNormal:
                        creep.hp -= 100;
                        break;
                    case kSplash:
                        creep.hp -= 200;
                        break;
                    case kFrost:
                        creep.hp -= 50;
                        break;
                    default:
                        break;
                }
                
                creep.hp--;
				
                if (creep.hp <= 0) {
                    DataModel *m = [DataModel getModel];
                    [targetsToDelete addObject:target];
                    m._currentScore += creep.score;
                    m._currentCoins += creep.coins;
                    [self displayScoreCoins];
                }
                break;
                
			}						
		}
		
		for (CCSprite *target in targetsToDelete) {
			[m._targets removeObject:target];
			[self removeChild:target cleanup:YES];									
		}
        
		[targetsToDelete release];
	}
	
	for (CCSprite *projectile in projectilesToDelete) {
		[m._projectiles removeObject:projectile];
		[self removeChild:projectile cleanup:YES];
	}
	[projectilesToDelete release];
    
    if([self levelComplete])
    {
        [self pauseGame];
    }
}

-(bool)levelComplete
{
    DataModel *m = [DataModel getModel];
    bool lc = ((self.currentWave == m._waves.count - 1) && ([self getCurrentWave].totalCreeps <= 0) && (m._targets.count <= 0));
    if(lc)
    {
        NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
        switch (self.currentLevel) {
            case 1:
                [defaults setBool:true forKey:@"isLevel1Complete"];
                break;
            case 2:
                [defaults setBool:true forKey:@"isLevel2Complete"];
                break;
            case 3:
                [defaults setBool:true forKey:@"isLevel3Complete"];
                break;
            default:
                break;
        }
        [defaults synchronize];
    }
    return lc;
}

- (CGPoint)boundLayerPos:(CGPoint)newPos {
    CGSize winSize = [CCDirector sharedDirector].winSize;
    CGPoint retval = newPos;
    retval.x = MIN(retval.x, 0);
    retval.x = MAX(retval.x, -_tileMap.contentSize.width+winSize.width); 
    retval.y = MIN(0, retval.y);
    retval.y = MAX(-_tileMap.contentSize.height+winSize.height, retval.y); 
    return retval;
}

- (void)handlePanFrom:(UIPanGestureRecognizer *)recognizer {
    
    if (recognizer.state == UIGestureRecognizerStateBegan) {    
        
        CGPoint touchLocation = [recognizer locationInView:recognizer.view];
        touchLocation = [[CCDirector sharedDirector] convertToGL:touchLocation];
        touchLocation = [self convertToNodeSpace:touchLocation];                
        
    } else if (recognizer.state == UIGestureRecognizerStateChanged) {    
        
        CGPoint translation = [recognizer translationInView:recognizer.view];
        translation = ccp(translation.x, -translation.y);
        CGPoint newPos = ccpAdd(self.position, translation);
        self.position = [self boundLayerPos:newPos];  
        [recognizer setTranslation:CGPointZero inView:recognizer.view];    
        
    } else if (recognizer.state == UIGestureRecognizerStateEnded) {
        
		float scrollDuration = 0.2;
		CGPoint velocity = [recognizer velocityInView:recognizer.view];
		CGPoint newPos = ccpAdd(self.position, ccpMult(ccp(velocity.x, velocity.y * -1), scrollDuration));
		newPos = [self boundLayerPos:newPos];
        
		[self stopAllActions];
		CCMoveTo *moveTo = [CCMoveTo actionWithDuration:scrollDuration position:newPos];            
		[self runAction:[CCEaseOut actionWithAction:moveTo rate:1]];            
        
    }        
}

// on "dealloc" you need to release all your retained objects
- (void) dealloc
{
	[super dealloc];
}


-(void)pauseGame {
    if(self.isPause)
    {
        return;
    }
    
    self.isPause=true;
    [[CCDirector sharedDirector] pause];
    DataModel *m = [DataModel getModel];
    
    CGSize winSize = [CCDirector sharedDirector].winSize;
    float fontSize = 20.0;
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
        fontSize *= 2;
    }
    
    if([self levelComplete])
    {
        PauseMenuLayer = [CCLayerColor layerWithColor: ccc4(0, 0, 255, 125) width: 270 height: 75];
        PauseMenuLayer.position = ccp(winSize.width*0.2, winSize.height*0.38);
        [self addChild: PauseMenuLayer z:8];
        
        CCLabelBMFont *opLabel = [CCLabelTTF labelWithString:@"YOU WIN" fontName:@"AmericanTypewriter-Bold" fontSize:fontSize];
        CCMenuItemLabel *op = [CCMenuItemLabel itemWithLabel:opLabel target:self selector:nil];
        [op setTag:1];
        
        CCLabelBMFont *nextLabel = [CCLabelTTF labelWithString:@"CONTINUE" fontName:@"AmericanTypewriter-Bold" fontSize:fontSize];
        CCMenuItemLabel *next = [CCMenuItemLabel itemWithLabel:nextLabel target:self selector:@selector(nextGame)];
        [next setTag:2];
        
        CCLabelBMFont *mainMenuLabel = [CCLabelTTF labelWithString:@"MENU" fontName:@"AmericanTypewriter-Bold" fontSize:fontSize];
        CCMenuItemLabel *mainMenu = [CCMenuItemLabel itemWithLabel:mainMenuLabel target:self selector:@selector(gotomainMenu)];
        [mainMenu setTag:3];
        
        
        pauseMenu = [CCMenu menuWithItems:op, next, mainMenu, nil];
        [pauseMenu alignItemsInColumns:
         [NSNumber numberWithUnsignedInt:1],
         [NSNumber numberWithUnsignedInt:2],
         nil
         ];

    }
    else if(m._currentHP>0)
    {
        
        PauseMenuLayer = [CCLayerColor layerWithColor: ccc4(0, 0, 255, 125) width: 215 height: 75];
        PauseMenuLayer.position = ccp(winSize.width*0.28, winSize.height*0.38);
        [self addChild: PauseMenuLayer z:8];
        
        CCLabelBMFont *resumeLabel = [CCLabelTTF labelWithString:@"RESUME" fontName:@"AmericanTypewriter-Bold" fontSize:fontSize];
        CCMenuItemLabel *resume = [CCMenuItemLabel itemWithLabel:resumeLabel target:self selector:@selector(resumeGame)];
        [resume setTag:1];
    
        CCLabelBMFont *mainMenuLabel = [CCLabelTTF labelWithString:@"MENU" fontName:@"AmericanTypewriter-Bold" fontSize:fontSize];
        CCMenuItemLabel *mainMenu = [CCMenuItemLabel itemWithLabel:mainMenuLabel target:self selector:@selector(gotomainMenu)];
        [mainMenu setTag:2];
        
        
        pauseMenu = [CCMenu menuWithItems:resume, mainMenu, nil];
        
        
        [pauseMenu alignItemsHorizontallyWithPadding:winSize.height * 0.059f];
    } 
    else
    { 
        PauseMenuLayer = [CCLayerColor layerWithColor: ccc4(0, 0, 255, 125) width: 270 height: 75];
        PauseMenuLayer.position = ccp(winSize.width*0.2, winSize.height*0.38);
        [self addChild: PauseMenuLayer z:8];
        
        CCLabelBMFont *opLabel = [CCLabelTTF labelWithString:@"GAME OVER" fontName:@"AmericanTypewriter-Bold" fontSize:fontSize];
        CCMenuItemLabel *op = [CCMenuItemLabel itemWithLabel:opLabel target:self selector:nil];
        [op setTag:1];
        
        CCLabelBMFont *restartLabel = [CCLabelTTF labelWithString:@"RESTART" fontName:@"AmericanTypewriter-Bold" fontSize:fontSize];
        CCMenuItemLabel *restart = [CCMenuItemLabel itemWithLabel:restartLabel target:self selector:@selector(restartGame)];
        [restart setTag:2];
        
        CCLabelBMFont *mainMenuLabel = [CCLabelTTF labelWithString:@"MENU" fontName:@"AmericanTypewriter-Bold" fontSize:fontSize];
        CCMenuItemLabel *mainMenu = [CCMenuItemLabel itemWithLabel:mainMenuLabel target:self selector:@selector(gotomainMenu)];
        [mainMenu setTag:3];
        
        
        pauseMenu = [CCMenu menuWithItems:op, restart, mainMenu, nil];
        [pauseMenu alignItemsInColumns:
         [NSNumber numberWithUnsignedInt:1],
         [NSNumber numberWithUnsignedInt:2],
         nil
         ];
        
        NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
        if(m._currentScore>self.maxScore)
            [defaults setDouble:m._currentScore forKey:@"HighScores"];
    }
    
    id moveAction = [CCMoveTo actionWithDuration:0.5f 
                                        position:ccp(winSize.width * 1.0f, 
                                                     winSize.height/2)];
    id moveEffect = [CCEaseIn actionWithAction:moveAction rate:1.0f];
    [pauseMenu runAction:moveEffect];
    
    [self addChild:pauseMenu z:10];

}

-(void)restartGame {
    [self resumeGame];
    [self removeChild:gameHUD cleanup:YES];
    [[GameManager sharedGameManager] runSceneWithID:kGameLevel1];
}

-(void)nextGame {
    [self resumeGame];
    [self removeChild:gameHUD cleanup:YES];
    DataModel *m = [DataModel getModel];
    m._levelComplete = true;
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    switch (self.currentLevel) {
        case 1:
            [[GameManager sharedGameManager] runSceneWithID:kGameLevel2];
            break;
        case 2:
            [[GameManager sharedGameManager] runSceneWithID:kGameLevel3];
            break;
        case 3:
            // Get the stored data before the view loads
            
            [m reset];
            if(m._currentScore>self.maxScore)
                [defaults setDouble:m._currentScore forKey:@"HighScores"];            
            [[GameManager sharedGameManager] runSceneWithID:kHighScoresScene];
            break;
        default:
            break;
    }
}

-(void)resumeGame {
    self.isPause=false;
    [self removeChild:pauseMenu cleanup:YES];
    [self removeChild:PauseMenuLayer cleanup:YES];
    //[[SimpleAudioEngine sharedEngine] playBackgroundMusic:@"backgroundmusic.mp3"];
    [[CCDirector sharedDirector] resume];
}

-(void)gotomainMenu {
    [self resumeGame];
    self.isPause=true;
    [self removeChild:gameHUD cleanup:YES];
    [[GameManager sharedGameManager] runSceneWithID:kMainMenuScene];
}


@end
