//
//  OptionsLayer.m
//  SpaceViking
//

#import "OptionsLayer.h"


@implementation OptionsLayer
-(void)returnToMainMenu {
	[[GameManager sharedGameManager] runSceneWithID:kMainMenuScene];
}

-(void)musicTogglePressed {
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	if ([[GameManager sharedGameManager] isMusicON]) {
		CCLOG(@"OptionsLayer-> Turning Game Music OFF");
		[[GameManager sharedGameManager] setIsMusicON:NO];
        [[GameManager sharedGameManager] stopBackgroundTrack];
	} else {
		CCLOG(@"OptionsLayer-> Turning Game Music ON");
		[[GameManager sharedGameManager] setIsMusicON:YES];
        [[GameManager sharedGameManager] playBackgroundTrack:BACKGROUND_TRACK_MAIN_MENU];
	}
}

-(void)SFXTogglePressed {
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	if ([[GameManager sharedGameManager] isSoundEffectsON]) {
		CCLOG(@"OptionsLayer-> Turning Sound Effects OFF");
		[[GameManager sharedGameManager] setIsSoundEffectsON:NO];
	} else {
		CCLOG(@"OptionsLayer-> Turning Sound Effects ON");
		[[GameManager sharedGameManager] setIsSoundEffectsON:YES];
	}
}

-(void)difficultyTogglePressed {
    if([[GameManager sharedGameManager] difficulty] == 3)
        [[GameManager sharedGameManager] setDifficulty:0];
    else
        [[GameManager sharedGameManager] setDifficulty:[[GameManager sharedGameManager] difficulty]+1];
}

-(id)init {
	self = [super init];
	if (self != nil) {
		CGSize screenSize = [CCDirector sharedDirector].winSize; 
		
		CCSprite *background = [CCSprite spriteWithFile:@"GenericMenuBackground.png"];
		[background setPosition:ccp(screenSize.width/2, screenSize.height/2)];
		[self addChild:background];
		
		
		CCLabelBMFont *musicOnLabelText = [CCLabelBMFont labelWithString:@"Music ON" fntFile:@"VikingSpeechFont64.fnt"];
		CCLabelBMFont *musicOffLabelText = [CCLabelBMFont labelWithString:@"Music OFF" fntFile:@"VikingSpeechFont64.fnt"];
		CCLabelBMFont *SFXOnLabelText = [CCLabelBMFont labelWithString:@"Sound Effects ON" fntFile:@"VikingSpeechFont64.fnt"];
		CCLabelBMFont *SFXOffLabelText = [CCLabelBMFont labelWithString:@"Sound Effects OFF" fntFile:@"VikingSpeechFont64.fnt"];
        CCLabelBMFont *easyLabelText = [CCLabelBMFont labelWithString:@"Difficulty Easy" fntFile:@"VikingSpeechFont64.fnt"];
		CCLabelBMFont *normalLabelText = [CCLabelBMFont labelWithString:@"Difficulty Normal" fntFile:@"VikingSpeechFont64.fnt"];
        CCLabelBMFont *hardLabelText = [CCLabelBMFont labelWithString:@"Difficulty Hard" fntFile:@"VikingSpeechFont64.fnt"];
		CCLabelBMFont *veryhardLabelText = [CCLabelBMFont labelWithString:@"Difficulty Very Hard" fntFile:@"VikingSpeechFont64.fnt"];
		
		CCMenuItemLabel *musicOnLabel = [CCMenuItemLabel itemWithLabel:musicOnLabelText target:self selector:nil];
		CCMenuItemLabel *musicOffLabel = [CCMenuItemLabel itemWithLabel:musicOffLabelText target:self selector:nil];
		CCMenuItemLabel *SFXOnLabel = [CCMenuItemLabel itemWithLabel:SFXOnLabelText target:self selector:nil];
		CCMenuItemLabel *SFXOffLabel = [CCMenuItemLabel itemWithLabel:SFXOffLabelText target:self selector:nil];
        CCMenuItemLabel *easyLabel = [CCMenuItemLabel itemWithLabel:easyLabelText target:self selector:nil];
		CCMenuItemLabel *normalLabel = [CCMenuItemLabel itemWithLabel:normalLabelText target:self selector:nil];
		CCMenuItemLabel *hardLabel = [CCMenuItemLabel itemWithLabel:hardLabelText target:self selector:nil];
		CCMenuItemLabel *veryhardLabel = [CCMenuItemLabel itemWithLabel:veryhardLabelText target:self selector:nil];

										 
		CCMenuItemToggle *musicToggle = [CCMenuItemToggle itemWithTarget:self 
																selector:@selector(musicTogglePressed) 
																   items:musicOnLabel,musicOffLabel,nil];
		
		CCMenuItemToggle *SFXToggle = [CCMenuItemToggle itemWithTarget:self 
																selector:@selector(SFXTogglePressed) 
																   items:SFXOnLabel,SFXOffLabel,nil];
		CCMenuItemToggle *difficultyToggle = [CCMenuItemToggle itemWithTarget:self 
                                                              selector:@selector(difficultyTogglePressed) 
                                                                 items:easyLabel,normalLabel,hardLabel,veryhardLabel,nil];
        
		CCLabelBMFont *backButtonLabel = [CCLabelBMFont labelWithString:@"Back" fntFile:@"VikingSpeechFont64.fnt"];
		CCMenuItemLabel	*backButton = [CCMenuItemLabel itemWithLabel:backButtonLabel target:self selector:@selector(returnToMainMenu)];
			
		CCMenu *optionsMenu = [CCMenu menuWithItems:musicToggle,
							   SFXToggle,
                               difficultyToggle,
							   backButton, nil];
		[optionsMenu alignItemsVerticallyWithPadding:60.0f];
		[optionsMenu setPosition:ccp(screenSize.width * 0.75f, screenSize.height/2)];
		[self addChild:optionsMenu];
        
        if ([[GameManager sharedGameManager] isMusicON] == NO) {
            [musicToggle setSelectedIndex:1]; // Music is OFF
        }
        
        if ([[GameManager sharedGameManager] isSoundEffectsON] == NO) {
            [SFXToggle setSelectedIndex:1]; // SFX are OFF
        }
        
        
        [difficultyToggle setSelectedIndex:[[GameManager sharedGameManager] difficulty]];
	}
	return self;
}
@end
